/*
 * Copyright (c) 2001, 2002 The XDoclet team
 * All rights reserved.
 */
package xdoclet.modules.objectweb.jonas.ejb;

import java.util.*;

import xjavadoc.*;
import xdoclet.XDocletException;
import xdoclet.modules.ejb.EjbTagsHandler;
import xdoclet.modules.ejb.dd.RelationTagsHandler;
import xdoclet.util.Translator;

/**
 * This tag handler handles tags needed for JOnAS relations
 *
 * @author               <a href="mailto:aslak.nospam@users.sf.net">Aslak Helles?y</a>
 * @created              12 mei 2002
 * @modifier             Camilleri Jerome : adaptation for JOnAS relation tag
 * @xdoclet.taghandler   namespace="JOnASEjbRel"
 * @version              $Revision$
 */
public class JonasRelationTagsHandler extends RelationTagsHandler
{
    private Collection columnMapTags = null;
    private Iterator columnMapTagIterator = null;
    private XClass  pkClass;
    private String  currentKeyColumn = null;
    private String  currentForeignKeyColumn = null;

    /**
     * Describe what the method does
     *
     * @param template              Describe what the parameter does
     * @param left
     * @exception XDocletException  Describe the exception
     */
    public void forAllColumnMaps(String template, boolean left) throws XDocletException
    {
        XMethod methodA;
        XMethod methodB;
        XClass classA;
        XClass classB;

        if (left) {
            methodA = currentRelation.getLeftMethod();
            methodB = currentRelation.getRightMethod();
            classA = currentRelation.getLeft();
            classB = currentRelation.getRight();
        }
        else {
            methodA = currentRelation.getRightMethod();
            methodB = currentRelation.getLeftMethod();
            classA = currentRelation.getRight();
            classB = currentRelation.getLeft();
        }

        if (methodA != null) {
            // bidirectional
            columnMapTags = methodA.getDoc().getTags("jonas.field-mapping");

            columnMapTagIterator = columnMapTags.iterator();
            pkClass = classB;
            if (pkClass == null) {
                pkClass = EjbTagsHandler.getEjb(methodA.getDoc().getTagAttributeValue("ejb.relation", "target-ejb"));
            }
            if (columnMapTags.size() == 0) {
                throw new XDocletException(currentRelation.getRightMethod() + " should have at least one @jonas:target-jdbc-field tag");
            }
        }
        else {
            // unidirectional
            columnMapTags = methodB.getDoc().getTags("jonas.target-field-mapping");

            columnMapTagIterator = columnMapTags.iterator();
            pkClass = classB;
            if (columnMapTags.size() == 0) {
                throw new XDocletException(currentRelation.getLeftMethod() + " should have at least one @jonas:target-jdbc-field tag");
            }
        }
        while (columnMapTagIterator.hasNext()) {

            XTag columnMapTag = (XTag) columnMapTagIterator.next();

            currentKeyColumn = columnMapTag.getAttributeValue("key-jdbc-name");
            currentForeignKeyColumn = columnMapTag.getAttributeValue("foreign-key-jdbc-name");
            generate(template);
        }
    }

    public void forAllLeftColumnMaps(String template) throws XDocletException
    {
        forAllColumnMaps(template, true);
    }

    /**
     * Describe what the method does
     *
     * @param template              Describe what the parameter does
     * @exception XDocletException  Describe the exception
     */
    public void forAllRightColumnMaps(String template) throws XDocletException
    {
        forAllColumnMaps(template, false);
    }

    /**
     * Makes sure the column-map tags are on the right side and target-column-map tags left
     */
    public void ensureColumnMapTagsRight()
    {
        boolean rightHasTargetColumnMapTags = false;
        boolean leftHasColumnMapTags = false;

        if (currentRelation.getLeftMethod() != null) {
            rightHasTargetColumnMapTags = currentRelation.getLeftMethod().getDoc().getTags("jonas.target-field-mapping").size() != 0;
        }
        if (currentRelation.getRightMethod() != null) {
            leftHasColumnMapTags = currentRelation.getRightMethod().getDoc().getTags("jonas.field-mapping").size() != 0;
        }
        if (rightHasTargetColumnMapTags || leftHasColumnMapTags) {
            currentRelation.swap();
        }
    }


    /**
     * Describe what the method does
     *
     * @param template              Describe what the parameter does
     * @exception XDocletException  Describe the exception
     */
    public void ifHasKeyColumn(String template) throws XDocletException
    {
        if (keyColumn() != null) {
            generate(template);
        }
    }

    /**
     * Describe what the method does
     *
     * @return                      Describe the return value
     * @exception XDocletException
     */
    public String keyColumn() throws XDocletException
    {
        if (currentKeyColumn == null) {
            // Not specified. Look on related bean
            if (columnMapTags.size() > 1) {
                throw new XDocletException("Can''t guess what is the related pk column when there is more than one jonas.jdbc-field");
            }

            Collection methods = pkClass.getMethods();

            for (Iterator i = methods.iterator(); i.hasNext(); ) {
                XMethod method = (XMethod) i.next();
                boolean isPk = method.getDoc().getTag("ejb.pk-field") != null;
                String tmpKeyColumn = method.getDoc().getTagAttributeValue("ejb.persistence", "column-name");

                // see that we only have one pk field
                if (isPk) {
                    if (currentKeyColumn != null) {
                        throw new XDocletException("The bean has a compound key. Please specify pk-field");
                    }
                    else {
                        currentKeyColumn = tmpKeyColumn;
                    }
                }
            }
            if (currentKeyColumn == null) {
                throw new XDocletException("The bean doesn''t specify ejb.persistence column-name");
            }
        }
        return currentKeyColumn;
    }

    /**
     * Describe what the method does
     *
     * @return   Describe the return value
     */
    public String foreignKeyColumn()
    {
        return currentForeignKeyColumn;
    }
}
